@@ -1,9 +1,6 @@
-/* -*- Mode: C; indent-tabs-mode:t ; c-basic-offset:8 -*- */
 /*
  * Synchronous I/O functions for libusb
- * Copyright © 2007-2008 Daniel Drake <dsd@gentoo.org>
- * Copyright © 2019 Nathan Hjelm <hjelmn@cs.unm.edu>
- * Copyright © 2019 Google LLC. All rights reserved.
+ * Copyright (C) 2007-2008 Daniel Drake <dsd@gentoo.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,55 +17,31 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include "libusbi.h"
-
+#include <config.h>
+#include <errno.h>
+#include <stdint.h>
+#include <stdlib.h>
 #include <string.h>
 
+#include "libusbi.h"
+
 /**
- * @defgroup libusb_syncio Synchronous device I/O
+ * @defgroup syncio Synchronous device I/O
  *
  * This page documents libusb's synchronous (blocking) API for USB device I/O.
  * This interface is easy to use but has some limitations. More advanced users
- * may wish to consider using the \ref libusb_asyncio "asynchronous I/O API" instead.
+ * may wish to consider using the \ref asyncio "asynchronous I/O API" instead.
  */
 
-static void LIBUSB_CALL sync_transfer_cb(struct libusb_transfer *transfer)
+static void ctrl_transfer_cb(struct libusb_transfer *transfer)
 {
-	usbi_dbg(TRANSFER_CTX(transfer), "actual_length=%d", transfer->actual_length);
-
 	int *completed = transfer->user_data;
 	*completed = 1;
-	/*
-	 * Right after setting 'completed', another thread might free the transfer, so don't
-	 * access it beyond this point. The instantiating thread (not necessarily the
-	 * current one) interprets the result and frees the transfer.
-	 */
+	usbi_dbg("actual_length=%d", transfer->actual_length);
+	/* caller interprets result and frees transfer */
 }
 
-static void sync_transfer_wait_for_completion(struct libusb_transfer *transfer)
-{
-	int r, *completed = transfer->user_data;
-	struct libusb_context *ctx = HANDLE_CTX(transfer->dev_handle);
-
-	while (!*completed) {
-		r = libusb_handle_events_completed(ctx, completed);
-		if (r < 0) {
-			if (r == LIBUSB_ERROR_INTERRUPTED)
-				continue;
-			usbi_err(ctx, "libusb_handle_events failed: %s, cancelling transfer and retrying",
-				 libusb_error_name(r));
-			libusb_cancel_transfer(transfer);
-			continue;
-		}
-		if (NULL == transfer->dev_handle) {
-			/* transfer completion after libusb_close() */
-			transfer->status = LIBUSB_TRANSFER_NO_DEVICE;
-			*completed = 1;
-		}
-	}
-}
-
-/** \ingroup libusb_syncio
+/** \ingroup syncio
  * Perform a USB control transfer.
  *
  * The direction of the transfer is inferred from the bmRequestType field of
@@ -86,35 +59,28 @@ static void sync_transfer_wait_for_completion(struct libusb_transfer *transfer)
  * (depending on direction bits within bmRequestType)
  * \param wLength the length field for the setup packet. The data buffer should
  * be at least this size.
- * \param timeout timeout (in milliseconds) that this function should wait
- * before giving up due to no response being received. For an unlimited
- * timeout, use value 0.
+ * \param timeout timeout (in millseconds) that this function should wait
+ * before giving up due to no response being received. For no timeout, use
+ * value 0.
  * \returns on success, the number of bytes actually transferred
- * \returns \ref LIBUSB_ERROR_TIMEOUT if the transfer timed out
- * \returns \ref LIBUSB_ERROR_PIPE if the control request was not supported by the
+ * \returns LIBUSB_ERROR_TIMEOUT if the transfer timed out
+ * \returns LIBUSB_ERROR_PIPE if the control request was not supported by the
  * device
- * \returns \ref LIBUSB_ERROR_NO_DEVICE if the device has been disconnected
- * \returns \ref LIBUSB_ERROR_BUSY if called from event handling context
- * \returns \ref LIBUSB_ERROR_INVALID_PARAM if the transfer size is larger than
- * the operating system and/or hardware can support (see \ref asynclimits)
+ * \returns LIBUSB_ERROR_NO_DEVICE if the device has been disconnected
  * \returns another LIBUSB_ERROR code on other failures
  */
-int API_EXPORTED libusb_control_transfer(libusb_device_handle *dev_handle,
+API_EXPORTED int libusb_control_transfer(libusb_device_handle *dev_handle,
 	uint8_t bmRequestType, uint8_t bRequest, uint16_t wValue, uint16_t wIndex,
 	unsigned char *data, uint16_t wLength, unsigned int timeout)
 {
-	struct libusb_transfer *transfer;
+	struct libusb_transfer *transfer = libusb_alloc_transfer(0);
 	unsigned char *buffer;
 	int completed = 0;
 	int r;
 
-	if (usbi_handling_events(HANDLE_CTX(dev_handle)))
-		return LIBUSB_ERROR_BUSY;
-
-	transfer = libusb_alloc_transfer(0);
 	if (!transfer)
 		return LIBUSB_ERROR_NO_MEM;
-
+	
 	buffer = malloc(LIBUSB_CONTROL_SETUP_SIZE + wLength);
 	if (!buffer) {
 		libusb_free_transfer(transfer);
@@ -127,7 +93,7 @@ int API_EXPORTED libusb_control_transfer(libusb_device_handle *dev_handle,
 		memcpy(buffer + LIBUSB_CONTROL_SETUP_SIZE, data, wLength);
 
 	libusb_fill_control_transfer(transfer, dev_handle, buffer,
-		sync_transfer_cb, &completed, timeout);
+		ctrl_transfer_cb, &completed, timeout);
 	transfer->flags = LIBUSB_TRANSFER_FREE_BUFFER;
 	r = libusb_submit_transfer(transfer);
 	if (r < 0) {
@@ -135,7 +101,19 @@ int API_EXPORTED libusb_control_transfer(libusb_device_handle *dev_handle,
 		return r;
 	}
 
-	sync_transfer_wait_for_completion(transfer);
+	while (!completed) {
+		r = libusb_handle_events(HANDLE_CTX(dev_handle));
+		if (r < 0) {
+			if (r == LIBUSB_ERROR_INTERRUPTED)
+				continue;
+			libusb_cancel_transfer(transfer);
+			while (!completed)
+				if (libusb_handle_events(HANDLE_CTX(dev_handle)) < 0)
+					break;
+			libusb_free_transfer(transfer);
+			return r;
+		}
+	}
 
 	if ((bmRequestType & LIBUSB_ENDPOINT_DIR_MASK) == LIBUSB_ENDPOINT_IN)
 		memcpy(data, libusb_control_transfer_get_data(transfer),
@@ -154,13 +132,6 @@ int API_EXPORTED libusb_control_transfer(libusb_device_handle *dev_handle,
 	case LIBUSB_TRANSFER_NO_DEVICE:
 		r = LIBUSB_ERROR_NO_DEVICE;
 		break;
-	case LIBUSB_TRANSFER_OVERFLOW:
-		r = LIBUSB_ERROR_OVERFLOW;
-		break;
-	case LIBUSB_TRANSFER_ERROR:
-	case LIBUSB_TRANSFER_CANCELLED:
-		r = LIBUSB_ERROR_IO;
-		break;
 	default:
 		usbi_warn(HANDLE_CTX(dev_handle),
 			"unrecognised status code %d", transfer->status);
@@ -171,23 +142,27 @@ int API_EXPORTED libusb_control_transfer(libusb_device_handle *dev_handle,
 	return r;
 }
 
+static void bulk_transfer_cb(struct libusb_transfer *transfer)
+{
+	int *completed = transfer->user_data;
+	*completed = 1;
+	usbi_dbg("actual_length=%d", transfer->actual_length);
+	/* caller interprets results and frees transfer */
+}
+
 static int do_sync_bulk_transfer(struct libusb_device_handle *dev_handle,
 	unsigned char endpoint, unsigned char *buffer, int length,
 	int *transferred, unsigned int timeout, unsigned char type)
 {
-	struct libusb_transfer *transfer;
+	struct libusb_transfer *transfer = libusb_alloc_transfer(0);
 	int completed = 0;
 	int r;
 
-	if (usbi_handling_events(HANDLE_CTX(dev_handle)))
-		return LIBUSB_ERROR_BUSY;
-
-	transfer = libusb_alloc_transfer(0);
 	if (!transfer)
 		return LIBUSB_ERROR_NO_MEM;
 
 	libusb_fill_bulk_transfer(transfer, dev_handle, endpoint, buffer, length,
-		sync_transfer_cb, &completed, timeout);
+		bulk_transfer_cb, &completed, timeout);
 	transfer->type = type;
 
 	r = libusb_submit_transfer(transfer);
@@ -196,11 +171,21 @@ static int do_sync_bulk_transfer(struct libusb_device_handle *dev_handle,
 		return r;
 	}
 
-	sync_transfer_wait_for_completion(transfer);
-
-	if (transferred)
-		*transferred = transfer->actual_length;
+	while (!completed) {
+		r = libusb_handle_events(HANDLE_CTX(dev_handle));
+		if (r < 0) {
+			if (r == LIBUSB_ERROR_INTERRUPTED)
+				continue;
+			libusb_cancel_transfer(transfer);
+			while (!completed)
+				if (libusb_handle_events(HANDLE_CTX(dev_handle)) < 0)
+					break;
+			libusb_free_transfer(transfer);
+			return r;
+		}
+	}
 
+	*transferred = transfer->actual_length;
 	switch (transfer->status) {
 	case LIBUSB_TRANSFER_COMPLETED:
 		r = 0;
@@ -217,10 +202,6 @@ static int do_sync_bulk_transfer(struct libusb_device_handle *dev_handle,
 	case LIBUSB_TRANSFER_NO_DEVICE:
 		r = LIBUSB_ERROR_NO_DEVICE;
 		break;
-	case LIBUSB_TRANSFER_ERROR:
-	case LIBUSB_TRANSFER_CANCELLED:
-		r = LIBUSB_ERROR_IO;
-		break;
 	default:
 		usbi_warn(HANDLE_CTX(dev_handle),
 			"unrecognised status code %d", transfer->status);
@@ -231,7 +212,7 @@ static int do_sync_bulk_transfer(struct libusb_device_handle *dev_handle,
 	return r;
 }
 
-/** \ingroup libusb_syncio
+/** \ingroup syncio
  * Perform a USB bulk transfer. The direction of the transfer is inferred from
  * the direction bits of the endpoint address.
  *
@@ -248,7 +229,7 @@ static int do_sync_bulk_transfer(struct libusb_device_handle *dev_handle,
  * underlying O/S requirements, meaning that the timeout may expire after
  * the first few chunks have completed. libusb is careful not to lose any data
  * that may have been transferred; do not assume that timeout conditions
- * indicate a complete lack of I/O. See \ref asynctimeout for more details.
+ * indicate a complete lack of I/O.
  *
  * \param dev_handle a handle for the device to communicate with
  * \param endpoint the address of a valid endpoint to communicate with
@@ -257,34 +238,29 @@ static int do_sync_bulk_transfer(struct libusb_device_handle *dev_handle,
  * \param length for bulk writes, the number of bytes from data to be sent. for
  * bulk reads, the maximum number of bytes to receive into the data buffer.
  * \param transferred output location for the number of bytes actually
- * transferred. Since version 1.0.21 (\ref LIBUSB_API_VERSION >= 0x01000105),
- * it is legal to pass a NULL pointer if you do not wish to receive this
- * information.
- * \param timeout timeout (in milliseconds) that this function should wait
- * before giving up due to no response being received. For an unlimited
- * timeout, use value 0.
+ * transferred.
+ * \param timeout timeout (in millseconds) that this function should wait
+ * before giving up due to no response being received. For no timeout, use
+ * value 0.
  *
  * \returns 0 on success (and populates <tt>transferred</tt>)
- * \returns \ref LIBUSB_ERROR_TIMEOUT if the transfer timed out (and populates
+ * \returns LIBUSB_ERROR_TIMEOUT if the transfer timed out (and populates
  * <tt>transferred</tt>)
- * \returns \ref LIBUSB_ERROR_PIPE if the endpoint halted
- * \returns \ref LIBUSB_ERROR_OVERFLOW if the device offered more data, see
- * \ref libusb_packetoverflow
- * \returns \ref LIBUSB_ERROR_NO_DEVICE if the device has been disconnected
- * \returns \ref LIBUSB_ERROR_BUSY if called from event handling context
- * \returns \ref LIBUSB_ERROR_INVALID_PARAM if the transfer size is larger than
- * the operating system and/or hardware can support (see \ref asynclimits)
+ * \returns LIBUSB_ERROR_PIPE if the endpoint halted
+ * \returns LIBUSB_ERROR_OVERFLOW if the device offered more data, see
+ * \ref packetoverflow
+ * \returns LIBUSB_ERROR_NO_DEVICE if the device has been disconnected
  * \returns another LIBUSB_ERROR code on other failures
  */
-int API_EXPORTED libusb_bulk_transfer(libusb_device_handle *dev_handle,
-	unsigned char endpoint, unsigned char *data, int length,
-	int *transferred, unsigned int timeout)
+API_EXPORTED int libusb_bulk_transfer(struct libusb_device_handle *dev_handle,
+	unsigned char endpoint, unsigned char *data, int length, int *transferred,
+	unsigned int timeout)
 {
 	return do_sync_bulk_transfer(dev_handle, endpoint, data, length,
 		transferred, timeout, LIBUSB_TRANSFER_TYPE_BULK);
 }
 
-/** \ingroup libusb_syncio
+/** \ingroup syncio
  * Perform a USB interrupt transfer. The direction of the transfer is inferred
  * from the direction bits of the endpoint address.
  *
@@ -301,7 +277,7 @@ int API_EXPORTED libusb_bulk_transfer(libusb_device_handle *dev_handle,
  * underlying O/S requirements, meaning that the timeout may expire after
  * the first few chunks have completed. libusb is careful not to lose any data
  * that may have been transferred; do not assume that timeout conditions
- * indicate a complete lack of I/O. See \ref asynctimeout for more details.
+ * indicate a complete lack of I/O.
  *
  * The default endpoint bInterval value is used as the polling interval.
  *
@@ -312,28 +288,24 @@ int API_EXPORTED libusb_bulk_transfer(libusb_device_handle *dev_handle,
  * \param length for bulk writes, the number of bytes from data to be sent. for
  * bulk reads, the maximum number of bytes to receive into the data buffer.
  * \param transferred output location for the number of bytes actually
- * transferred. Since version 1.0.21 (\ref LIBUSB_API_VERSION >= 0x01000105),
- * it is legal to pass a NULL pointer if you do not wish to receive this
- * information.
- * \param timeout timeout (in milliseconds) that this function should wait
- * before giving up due to no response being received. For an unlimited
- * timeout, use value 0.
+ * transferred.
+ * \param timeout timeout (in millseconds) that this function should wait
+ * before giving up due to no response being received. For no timeout, use
+ * value 0.
  *
  * \returns 0 on success (and populates <tt>transferred</tt>)
- * \returns \ref LIBUSB_ERROR_TIMEOUT if the transfer timed out
- * \returns \ref LIBUSB_ERROR_PIPE if the endpoint halted
- * \returns \ref LIBUSB_ERROR_OVERFLOW if the device offered more data, see
- * \ref libusb_packetoverflow
- * \returns \ref LIBUSB_ERROR_NO_DEVICE if the device has been disconnected
- * \returns \ref LIBUSB_ERROR_BUSY if called from event handling context
- * \returns \ref LIBUSB_ERROR_INVALID_PARAM if the transfer size is larger than
- * the operating system and/or hardware can support (see \ref asynclimits)
+ * \returns LIBUSB_ERROR_TIMEOUT if the transfer timed out
+ * \returns LIBUSB_ERROR_PIPE if the endpoint halted
+ * \returns LIBUSB_ERROR_OVERFLOW if the device offered more data, see
+ * \ref packetoverflow
+ * \returns LIBUSB_ERROR_NO_DEVICE if the device has been disconnected
  * \returns another LIBUSB_ERROR code on other error
  */
-int API_EXPORTED libusb_interrupt_transfer(libusb_device_handle *dev_handle,
-	unsigned char endpoint, unsigned char *data, int length,
-	int *transferred, unsigned int timeout)
+API_EXPORTED int libusb_interrupt_transfer(
+	struct libusb_device_handle *dev_handle, unsigned char endpoint,
+	unsigned char *data, int length, int *transferred, unsigned int timeout)
 {
 	return do_sync_bulk_transfer(dev_handle, endpoint, data, length,
 		transferred, timeout, LIBUSB_TRANSFER_TYPE_INTERRUPT);
 }
+
